<?
/*
  CyberPlus Paiement ATOS/SIPS for Banque Populaire
  (http://www.atos-group.com/sips/)

  This module has been developed to fit latest osCommerce product requirement
  by e-network (http://www.e-network.fr/).

  It is compatible with version 2.2 MS1 or higher of the osCommerce
  product.

  You MUST have the ATOS/SIPS API version 5.00 or higher.
  This API has been released on september 2002.

  However, it is not compatible with 'old' osCommerce snapshot release
  2.2. To know, if this module is adapted to your package, check the
  following point :

  (1) files checkout_payment_address.php and checkout_shipping_address.php
      exist in your environment,

  (2) the function 'selection' (of another paiment module) returns an array
      instead of a string,

  (3) if you install this module from the administration area, and not able
      to see it when you try to purchase an order, this module is probably
      not adapted to your environment.

  Originally written by D. Nazim (a.k.a. vanadium2 <vanadium2@yahoo.com>).

  Modified and enhanced to fit new purchase process of the osCommerce 2.2 MS1
  and new ATOS/SIPS binaries by S. Guiboud-Ribaud <devteam@e-network.fr>.

  Copyright (C) 2002-2003, osCommerce
  Copyright (C) 2002-2003, e-network

  Release under the GNU General Public License.
*/

  define('DIR_FS_ATOS', DIR_FS_CATALOG . 'atos/');

  class atos {
    var $code, $description, $enabled, $title, $email_footer, $production, $currencies, $languages;

    // class constructor
    // The list of supported currencies
    function atos() {
      global $order;

      $this->currencies   = array(
				  'EUR' => '978',
				  'USD' => '840',
				  'CHF' => '756',
				  'GBP' => '826',
				  'CAD' => '124',
				  'JPY' => '392',
				  'MXP' => '484',
				  'TRL' => '792',
				  'AUD' => '036',
				  'NZD' => '554',
				  'NOK' => '578',
				  'BRC' => '986',
				  'ARP' => '032',
				  'KHR' => '116',
				  'TWD' => '901',
				  'SEK' => '752',
				  'DKK' => '208',
				  'KRW' => '410',
				  'SGD' => '702',
				  );
      $this->languages = array(
			       'english' => 'en',
			       'german'  => 'de',
			       'espanol' => 'es',
			       'french'  => 'fr',
			       );

      $this->code         = 'atos';
      $this->description  = MODULE_PAYMENT_ATOS_TEXT_DESCRIPTION;
      $this->enabled      = MODULE_PAYMENT_ATOS_STATUS == 'True' ? true : false;;
      $this->title        = MODULE_PAYMENT_ATOS_TEXT_TITLE;
      $this->email_footer = MODULE_PAYMENT_ATOS_TEXT_EMAIL_FOOTER;
      $this->production   = MODULE_PAYMENT_ATOS_PRODUCTION_MODE == 'production' ? true : false;
      $this->sort_order   = MODULE_PAYMENT_ATOS_SORT_ORDER;

      if ((int)MODULE_PAYMENT_ATOS_ORDER_STATUS_ID > 0) {
        $this->order_status = MODULE_PAYMENT_ATOS_ORDER_STATUS_ID;
      }

      if (is_object($order))
	$this->update_status();
    }


    // class methods
    function update_status() {
      global $order;

      if ( ($this->enabled == true) && ((int)MODULE_PAYMENT_ATOS_ZONE > 0) ) {
        $check_flag = false;
        $check_query = tep_db_query("select zone_id from " . TABLE_ZONES_TO_GEO_ZONES . " where geo_zone_id = '" . MODULE_PAYMENT_ATOS_ZONE . "' and zone_country_id = '" . $order->billing['country']['id'] . "' order by zone_id");
        while ($check = tep_db_fetch_array($check_query)) {
          if ($check['zone_id'] < 1) {
            $check_flag = true;
            break;
          } elseif ($check['zone_id'] == $order->billing['zone_id']) {
            $check_flag = true;
            break;
          }
        }

        if ($check_flag == false) {
          $this->enabled = false;
        }
      }
    }

    function javascript_validation() {
      return false;
    }

    // selection function
    function selection() {
      $selection = array('id'     => $this->code,
			 'module' => $this->title);

      if (!$this->production)
	$selection['fields'] = array(array('title' => '<font class="highlight">' . MODULE_PAYMENT_ATOS_TEXT_TITLE_DEMOMODE . '</font>',
					   'field' => ''));

      return $selection;
    }

    // pre_confirmation_check
    function pre_confirmation_check() {
      return false;
    }

    // confirmation
    function confirmation() {
      global $HTTP_POST_VARS, $HTTP_SERVER_VARS, $order, $currency;

      $sips = $this->makeRequest($order->info['total'], $currency);

      if (!$this->production)
	$this->sendNotification("ATOS Request Made", $sips);

      if ($sips['error']) {
	$this->error   = 1;
	$this->message = MODULE_PAYMENT_ATOS_CALL_REQUEST_ERROR . ': ' . $sips['command'] . '<br>' . $sips['result'];
      } else {
	$regs = array();
	$this->error   = 0;
	$this->message = $sips['message'];
	if (eregi('<form [^>]*action="([^"]*)"[^>]*>(.*)</form>', $sips['message'], $regs)) {
	  $this->message = $regs[2];
	  $this->form_action_url = $regs[1];
	  $this->form_submit     = '';
	} else {
	  $this->error = 1;
	  $this->message = MODULE_PAYMENT_ATOS_CALL_REQUEST_ERROR;
	}

      }
      if ($this->error) {
        /* An internal error occurs: inform the customer and invite him to contact
           our service and team */
        // FIXME
        $this->sendNotification('ATOS Request Error', $sips);

	$this->form_action_url = FILENAME_CHECKOUT_PAYMENT;
        $this->form_submit     = tep_image_submit('button_back.gif', IMAGE_BUTTON_BACK);
      }
      return false;
    }

    function process_button() {
      if ($this->error)
	return '<table border="0" width="100%" cellspacing="0" cellpadding="2"><tr><td class="highlight" align="center">'
	  . $this->message
	  . '</td></tr></table><br>';

      return $this->message;
    }

    function before_process() {
      global $order, $_SERVER, $HTTP_POST_VARS;

      if ($_SERVER['REMOTE_ADDR'] != MODULE_PAYMENT_ATOS_IP) {
	/* Ignore not allowed ip */
	$this->sendNotification('ATOS AutoResponse Status', '',
				$_SERVER['REMOTE_ADDR'] . " tries to connect to " . $GLOBALS['PHP_SELF']
				. '<br>Allowed ip is : ' . MODULE_PAYMENT_ATOS_IP);
	tep_redirect(tep_href_link(FILENAME_CHECKOUT_PAYMENT, '', 'SSL'));
      }

      /* check response data and code and update session information */
      $response = $this->decodeResponse($HTTP_POST_VARS['DATA']);
      if ($response['code'] != 0) {
	/* An internal error occurs: inform the customer and invite him to contact
         our service and team */
	// FIXME
	$this->sendNotification('ATOS AutoResponse Error', $response);
	tep_redirect(tep_href_link(FILENAME_CHECKOUT_PAYMENT, '', 'SSL'));
      }

      if ($response['merchant_id'] != MODULE_PAYMENT_ATOS_ID) {
	$this->sendNotification('ATOS AutoResponse Error',
				$response,
				'merchant_id differ:' . '<br>Merchant id expected = ' . MODULE_PAYMENT_ATOS_ID);
	tep_redirect(tep_href_link(FILENAME_CHECKOUT_PAYMENT, '', 'SSL'));
      }

      // Check whether response code is ok
      if ($response['response_code'] != '00')
        tep_redirect(tep_href_link(FILENAME_CHECKOUT_PAYMENT, '', 'SSL'));

      // Check whether the production mode is on or off
      if (!$this->production) {
        $this->sendNotification('ATOS checkout_process', $response, 'demo mode');
        tep_redirect(tep_href_link(FILENAME_CHECKOUT_PAYMENT,
				   'info_message=' . urlencode(MODULE_PAYMENT_ATOS_TEXT_CHECKOUT_DEMO), 'SSL', false));
      }

      /* Update credit card info and transaction id*/
      $GLOBALS['transaction_id'] = $response['transaction_id'];
      $constant = 'MODULE_PAYMENT_ATOS_PAYMENT_MEANS_' . $response['payment_means'];
      define($constant, $response['payment_means']);
      $GLOBALS['cc_type']        = constant($constant);
      $cc = split('\.', $response['card_number']);
      $GLOBALS['cc_number']      = $cc[0] . ' #### #### ##' . $cc[1];

      return false;
    }

    function after_process() {
      return false;
    }

    function output_error() {
      return false;
    }

    function check() {
      $check = tep_db_query("select configuration_value from " . TABLE_CONFIGURATION . " where configuration_key = 'MODULE_PAYMENT_ATOS_STATUS'");
      $check = tep_db_num_rows($check);
      return $check;
    }

    function install() {
      $install_atos_email  = STORE_OWNER_EMAIL_ADDRESS;

      $certificate = atos::FindACertificate();
      $install_merchant_id  = $certificate['id'];

      $install_parmcom_file = $certificate['parmcom'];
      if (!$install_parmcom_file || !file_exists(DIR_FS_ATOS . $install_parmcom_file)) {
	$install_parmcom_file = atos::FindAParmcomFile();
      }

      tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) values ('ACCEPTER le PAIEMENT ATOS-SIPS:', 'MODULE_PAYMENT_ATOS_STATUS', 'True', 'Voulez vous accepter les paiements ATOS?', '6', '1', 'tep_cfg_select_option(array(\'True\', \'False\'), ', now())");
      tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) values ('Identifiant de Commerant:', 'MODULE_PAYMENT_ATOS_ID', '" . $install_merchant_id . "', 'Votre ID marchand provenant d\'ATOS. A demander a votre banque.  Pour plus d\'information http://www.sips-atos.com.', '6', '2', 'atos::ConfigPullDownCertificates(', now())");
      tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) values ('Nom du fichier de paramtres gnrals:', 'MODULE_PAYMENT_ATOS_PARMCOM_FILE', '" . $install_parmcom_file . "', 'Choisissez le nom du fichier pour les paramtres gnrales.', '6', '4','atos::ConfigPullDownParmcomFiles(', now())");
      tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('<u>IMPORTANT</u> L\'IP autoris  donner la confirmation d\'achat:', 'MODULE_PAYMENT_ATOS_IP', '193.56.46.96', 'Entrer le No IP du serveur hbergeant le CGI qui fera l\'appel  votre serveur pour confirmer/infirmer l\'achat. Seul ce serveur est autoris  contacter votre serveur une fois la transaction effectue ou annule', '6', '4', now())");
      tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('Nom de l\'application pour construire la requte', 'MODULE_PAYMENT_ATOS_BIN_REQUEST', '" . DIR_FS_ATOS . "request', 'Chemin complet de l\'executable permettant de formatter la requte.', '6', '5', now())");
      tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('Nom de l\'application pour lire la rponse', 'MODULE_PAYMENT_ATOS_BIN_RESPONSE', '" . DIR_FS_ATOS . "response', 'Chemin complet de l\'executable pour dcoder la rponse envoye sur le script de rponse normale ou sur le script d\'auto-rponse', '6', '6', now())");
      tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('Nom du fichier pathfile', 'MODULE_PAYMENT_ATOS_PATHFILE', '', 'Pour laissez le module grer lui-mme le fichier pathfile, laissez ce champ vide (recommand dans la majorit des utilisations). Si vous voulez utiliser un fichier spcifique, alors saisissez le chemin complet pour ce fichier.', '6', '6', now())");
      tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) values ('Mise en production du module de paiement', 'MODULE_PAYMENT_ATOS_PRODUCTION_MODE', 'production', 'Si la valeur de ce champ vaut <tt>demo</tt>, les commandes client ne seront pas enregistres dans la base de donnes. Conserver cette valeur uniquement pour un mode dmonstration. Pour la mise en production, mettez ce champ  <tt>production</tt>', '6', '7', 'tep_cfg_select_option(array(\'demo\', \'production\'),', now())");
      tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('Adresse e-mail o envoyer les notifications (erreurs)', 'MODULE_PAYMENT_ATOS_EMAIL_FOR_NOTIFICATION', '" . $install_atos_email . "', 'Renseigner ce champ pour recevoir par email les erreurs survenues lors du traitement des transactions ATOS/SIPS', '6', '8', now())");
      tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('Ordre de tri', 'MODULE_PAYMENT_ATOS_SORT_ORDER', '0', 'Ordre de tri pour l\'affichage (trier par nombres croissants).', '6', '9', now())");
      tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, use_function, set_function, date_added) values ('Zone de Paiement', 'MODULE_PAYMENT_ATOS_ZONE', '0', 'Si une zone est slectionne, ce mode de paiement ne sera disponible uniquement pour cette zone.', '6', '10', 'tep_get_zone_class_title', 'tep_cfg_pull_down_zone_classes(', now())");
      tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, use_function, date_added) values ('tat initial de la commande', 'MODULE_PAYMENT_ATOS_ORDER_STATUS_ID', '0', 'Positionnez l\'tat initial de la commande, lorsque le client utilise ce mode de paiement', '6', '11', 'tep_cfg_pull_down_order_statuses(', 'tep_get_order_status_name', now())");
    }

    function remove() {
      tep_db_query("delete from " . TABLE_CONFIGURATION . " where configuration_key in ('" . implode("', '", $this->keys()) . "')");
    }

    function keys() {
      $keys = array(
		    'MODULE_PAYMENT_ATOS_STATUS',
		    'MODULE_PAYMENT_ATOS_ID',
		    'MODULE_PAYMENT_ATOS_PARMCOM_FILE',
		    'MODULE_PAYMENT_ATOS_IP',
		    'MODULE_PAYMENT_ATOS_BIN_REQUEST',
		    'MODULE_PAYMENT_ATOS_BIN_RESPONSE',
		    'MODULE_PAYMENT_ATOS_PATHFILE',
		    'MODULE_PAYMENT_ATOS_PRODUCTION_MODE',
		    'MODULE_PAYMENT_ATOS_EMAIL_FOR_NOTIFICATION',
		    'MODULE_PAYMENT_ATOS_SORT_ORDER',
		    'MODULE_PAYMENT_ATOS_ZONE',
		    'MODULE_PAYMENT_ATOS_ORDER_STATUS_ID'
		    );
      return $keys;
    }

    // Private methods

    // ----------------------------------------------------------------
    // Get the language code from the internal representation to the
    // value expected by ATOS/SIPS.
    // The list of supported languages are in the array 'languages;
    // set at the constructor level.
    function _getLanguageCode($language)
      {
	$value = $this->languages[$language];
	
	if ($value)
	  return $value;

	echo '<pre>';
	echo 'Sorry, language ', $language, ' not configured', "\n";
	print_r($this->languages);
	echo '</pre>';
	die;
      }

    // Get the currency code from the internal representation to the
    // representation expected by ATOS/SIPS.
    // The list of supported languages are in the array 'languages;
    // set at the constructor level.
    function _getCurrencyCode($currency)
      {
	$value = $this->currencies[$currency];
	
	if ($value)
	  return $value;
	
	echo '<pre>';
	echo 'Sorry, currency "', $currency, '" not configured', "\n";
	print_r($this->currencies);
	echo '</pre>';
	die;
      }

    // Get the currency unit factor.
    // This information is calculated using the number of decimal to
    // print for a currency, powered by 10.
    function _getCurrencyUnit($currency)
      {
	global $currencies;
	
	return pow(10, intval($currencies->currencies[$currency]['decimal_places']));
      }

    // Make the request by executing the atos_request binary.
    function makeRequest($amount, $currency)
      {
	global $customer_id, $HTTP_SERVER_VARS;
	global $language;
  
	$amount = sprintf("%03d", $amount * $this->_getCurrencyUnit($currency));

	$pathfile = $this->_getPathFileName();

	$command  = MODULE_PAYMENT_ATOS_BIN_REQUEST;
	$command .= " 'pathfile=" . $pathfile . "'";
	$command .= " merchant_id=" . MODULE_PAYMENT_ATOS_ID;
	$command .= " amount=" . $amount;
	$command .= " currency_code=" . $this->_getCurrencyCode(DEFAULT_CURRENCY);
	$command .= " language=" . $this->_getLanguageCode($language);
	$command .= " 'normal_return_url=" . tep_href_link('atos_response.php', tep_session_name() . '=' . tep_session_id(), 'SSL', false) . "'";
	$command .= " 'cancel_return_url=" . tep_href_link('atos_response.php', tep_session_name() . '=' . tep_session_id(), 'SSL', false) . "'";
	$command .= " 'automatic_response_url=" . tep_href_link(FILENAME_CHECKOUT_PROCESS, tep_session_name() . '=' . tep_session_id(), 'NONSSL', false) . "'";
	$command .= " customer_id=" . $customer_id;
	$command .= " customer_ip_address=" . $HTTP_SERVER_VARS['REMOTE_ADDR'];
	
	$sips_result = shell_exec("$command 2>&1");

	$sips = array();
	$sips_values     = split ("!", $sips_result);
	$sips['code']    = $sips_values[1];
	$sips['error']   = $sips_values[2];
	$sips['message'] = $sips_values[3];
	$sips['command'] = $command;
	$sips['result']  = $sips_result;
	
	if (!isset($sips['code']))
	  {
	    $sips['code']   = -1;
	    $sips['error']  = $sips_result;
	  }
	
	if ($sips['code'] != 0)
	  {
	    $sips['amount'] = $amount;
	    $sips['lang']   = $lang;
	    $sips['id']     = tep_session_id();
	  }
	
	return $sips;
      }

    // Decode DATA coming from the the response serveur
    function decodeResponse($data = '')
      {
	global $HTTP_POST_VARS;

	if (!$data)
	  $data = $HTTP_POST_VARS['DATA'];

	$pathfile = $this->_getPathFileName();

	$command  = MODULE_PAYMENT_ATOS_BIN_RESPONSE;
	$command .= " 'pathfile=" . $pathfile . "'";
	$command .= " message=$data";

	$sips_resp = shell_exec ( "$command 2>&1" );
  
	$hash = array ();
	$sips_resp = split ( "!", $sips_resp );
	list (,
	      $hash['code'],
	      $hash['error'],
	      $hash['merchant_id'],
	      $hash['merchant_country'],
	      $hash['amount'],
	      $hash['transaction_id'],
	      $hash['payment_means'],
	      $hash['transmission_date'],
	      $hash['payment_time'],
	      $hash['payment_date'],
	      $hash['response_code'],
	      $hash['payment_certificate'],
	      $hash['authorisation_id'],
	      $hash['currency_code'],
	      $hash['card_number'],
	      $hash['cvv_flag'],
	      $hash['cvv_response_code'],
	      $hash['bank_response_code'],
	      $hash['complementary_code'],
	      $hash['return_context'],
	      $hash['caddie'],
	      $hash['receipt_complement'],
	      $hash['merchant_language'],
	      $hash['language'],
	      $hash['customer_id'],
	      $hash['order_id'],
	      $hash['customer_email'],
	      $hash['customer_ip_address'],
	      $hash['capture_day'],
	      $hash['capture_mode'],
	      $hash['data']
	      ) = $sips_resp;
	
	if ($hash['code'] != 0)
	  {
	    $hash['command'] = $command;
	    $hash['output']  = $sips_resp;
	  }

	return $hash;
      }

    // Make the pathfile if it does not exists.
    function _getPathFileName()
      {
	if (MODULE_PAYMENT_ATOS_PATHFILE)
	  return MODULE_PAYMENT_ATOS_PATHFILE;

	$pathfile = DIR_FS_ATOS . 'pathfile.' . MODULE_PAYMENT_ATOS_PARMCOM_FILE;

	if (!file_exists($pathfile))
	  {
	    ob_start();
	    echo "#########################################################################\n";
	    echo "#\n";
	    echo "#	Pathfile \n";
	    echo "#\n";
	    echo "#	Liste fichiers parametres utiliss par le module de paiement\n";
	    echo "#\n";
	    echo "#########################################################################\n";
	    echo "\n";
	    echo "# ------------------------------------------------------------------------\n";
	    echo "# Chemin vers le rpertoire des logos depuis le web alias  \n";
	    echo "# Exemple pour le rpertoire www.merchant.com/cyberplus/payment/logo/\n";
	    echo "# indiquer:\n";
	    echo "# ------------------------------------------------------------------------\n";
	    echo "#\n";
	    echo "D_LOGO!",DIR_WS_IMAGES,"atos/!\n";
	    echo "#\n";
	    echo "#------------------------------------------------------------------------\n";
	    echo "#  Fichiers parametres lies a l'api cyberplus paiement	\n";
	    echo "#------------------------------------------------------------------------\n";
	    echo "#\n";
	    echo "# Certificat du commercant\n";
	    echo "#\n";
	    echo "F_CERTIFICATE!",DIR_FS_ATOS,"certif!\n";
	    echo "#\n";
	    echo "# Fichier paramtre commercant\n";
	    echo "#\n";
	    echo "F_PARAM!",DIR_FS_ATOS,"parmcom!\n";
	    echo "#\n";
	    echo "# Fichier des paramtres communs\n";
	    echo "#\n";
	    echo "F_DEFAULT!",DIR_FS_ATOS,MODULE_PAYMENT_ATOS_PARMCOM_FILE,"!\n";
	    echo "#\n";
	    echo "# --------------------------------------------------------------------------\n";
	    echo "# 	end of file\n";
	    echo "# --------------------------------------------------------------------------\n";
	    $pathfile_content = ob_get_contents();
	    ob_end_clean();
	
	    if ($fp = fopen($pathfile, 'w'))
	      {
		fputs($fp, $pathfile_content);
		fclose($fp);
	      }
	  }

	return $pathfile;
      }

    // Send an email to the ATOS Notification address

    function sendNotification($subject, $sips = '', $message = '')
      {
	global $HTTP_POST_VARS, $HTTP_GET_VARS;

	if (!MODULE_PAYMENT_ATOS_EMAIL_FOR_NOTIFICATION)
	  return;
	if (! $message)
	  $message = '<b>' . $GLOBALS['PHP_SELF'] . ': error at execution time' . "</b><br>\n";
	else
	  $message .= '<br>';
	
	$message .= 'List of parameters (POST):' . "\n";
	$message .= '<ul>';
	reset($HTTP_POST_VARS);
	while (list($key, $value) = each($HTTP_POST_VARS))
	  $message .= "<li><b>$key</b> = $value</li>\n";
	$message .= '</ul>';
	
	$message .= 'List of parameters (GET):' . "\n";
	$message .= '<ul>';
	reset($HTTP_GET_VARS);
	while (list($key, $value) = each($HTTP_GET_VARS))
	  $message .= "<li><b>$key</b> = $value</li>\n";
	$message .= '</ul>';
      
	if ($sips) {
	  $message .= 'List of return values by the call to request:' . "\n";
	  $message .= '<ul>';
	  while (list($key, $value) = each($sips))
	    $message .= "<li><b>$key</b> = $value</li>\n";
	  $message .= '</ul>';
	}

	tep_mail(STORE_NAME . ': ATOS', MODULE_PAYMENT_ATOS_EMAIL_FOR_NOTIFICATION, $subject,
		 $message,
		 STORE_NAME, STORE_OWNER_EMAIL_ADDRESS, '');
      }

    // Get the list of test certificates and information about them
    function GetPredefinedCertificates()
      {
	global $ATOS_PREDEFINED_CERTIFICATES;
	
	if (!$ATOS_PREDEFINED_CERTIFICATES) {
	  $ATOS_PREDEFINED_CERTIFICATES = array();
	  $ATOS_PREDEFINED_CERTIFICATES['013044876511111'] =
	    array(
		  'id'      => '013044876511111',
		  'parmcom' => 'parmcom.e-transactions',
		  'text'    => MODULE_PAYMENT_ATOS_CERTIFICATE_NAME_ETRANSACTION,
		  'test'    => true
		  );
	  $ATOS_PREDEFINED_CERTIFICATES['014213245611111'] =
	    array(
		  'id'      => '014213245611111',
		  'parmcom' => 'parmcom.sogenactif',
		  'text'    => MODULE_PAYMENT_ATOS_CERTIFICATE_NAME_SOGENACTIF,
		  'test'    => true
		  );
	  $ATOS_PREDEFINED_CERTIFICATES['038862749811111'] =
	    array(
		  'id'      => '038862749811111',
		  'parmcom' => 'parmcom.cyberplus',
		  'text'    => MODULE_PAYMENT_ATOS_CERTIFICATE_NAME_CYBERPLUS,
		  'test'    => true
		  );
	  $ATOS_PREDEFINED_CERTIFICATES['082584341411111'] =
	    array(
		  'id'      => '082584341411111',
		  'parmcom' => 'parmcom.mercanet',
		  'text'    => MODULE_PAYMENT_ATOS_CERTIFICATE_NAME_MERCANET,
		  'test'    => true
		  );
	}

      return $ATOS_PREDEFINED_CERTIFICATES;
    }

    // Get a certificate name
    function GetCertificateInfo($id)
      {
	$certificates = atos::GetPredefinedCertificates();

	if ($certificates[$id])
	  return $certificates[$id];

	return array(
		     'id'   => $id,
		     'text' => $id,
		     'test' => false
		     );
      }

    // Get the list of installed certificates
    function GetInstalledCertificates()
      {
	$certificates = array();
	if ($dir = @dir(DIR_FS_ATOS))
	  {
	    while ($file = $dir->read())
	      {
		$data = split('\.', $file);
		$n = sizeof($data) - 1;
		if ($data[0] == 'certif')
		  $certificates[] = atos::GetCertificateInfo($data[$n]);
	      }
	    sort($certificates);
	    $dir->close();
	  }
	return $certificates;
      }

    // Find a certificate to install
    function FindACertificate()
      {
	$certificate = null;
	foreach (atos::GetInstalledCertificates() as $current)
	  {
	    if (!$current['test'])
	      return $current;
	    if (!$certificate)
	      $certificate = $current;
	  }
	return $certificate;
      }

    // Get the list of installed certificates
    function GetInstalledParmcomFiles()
      {
	$parmcom_files = array();
	if ($dir = @dir(DIR_FS_ATOS))
	  {
	    while ($file = $dir->read())
	      {
		$data = split('\.', $file);
		if (($data[0] == 'parmcom') && !file_exists(DIR_FS_ATOS .'certif.fr.' . $data[1]))
		  $parmcom_files[] = array(
					   'id'   => $file,
					   'text' => $file
					   );
	      }
	    sort($parmcom_files);
	    $dir->close();
	  }
	return $parmcom_files;
      }

    // Find a parmcom file to install
    function FindAParmcomFile()
      {
	$parmcom_files = atos::GetInstalledParmcomFiles();
	return $parmcom_files[0]['id'];
      }

    // Pull down list to select certificate
    function ConfigPullDownCertificates($value, $key)
      {
	$certificates = atos::GetInstalledCertificates();
	
	$field_name = (($key) ? 'configuration[' . $key . ']' : 'configuration_value');
	
	if (sizeof($certificates) > 0)
	  return tep_draw_pull_down_menu($field_name, $certificates, $value);
	
	return tep_draw_input_field($field_name, $value['value']);
      }

    // Pull down list to select certificate
    function ConfigPullDownParmcomFiles($value, $key)
      {
	$parmcom_files = atos::GetInstalledParmcomFiles();
	
	$field_name = (($key) ? 'configuration[' . $key . ']' : 'configuration_value');
	
	if (sizeof($parmcom_files) > 0)
	  return tep_draw_pull_down_menu($field_name, $parmcom_files, $value);
	
	return tep_draw_input_field($field_name, $value['value']);
      }

  }
?>
